@@ -1,5 +1,7 @@
-from typing import Any, Mapping, Union
+from typing import Any, Mapping, Type, Union
 
+from google.protobuf import timestamp_pb2
+from google.protobuf.internal.enum_type_wrapper import EnumTypeWrapper
 from google.protobuf.message import Message
 
 from chalk.features._encoding.protobuf import (
@@ -19,6 +21,50 @@ def _is_protobuf_message(obj: Any) -> bool:
     )
 
 
+def _is_protobuf_enum_type(obj: Type) -> bool:
+    return isinstance(obj, EnumTypeWrapper) or (
+        hasattr(obj, "DESCRIPTOR") and hasattr(obj, "Name") and hasattr(obj, "Value") and hasattr(obj, "items")
+    )
+
+
+def proto_enum_value_to_name(value: Union[Underscore, int], proto_enum: EnumTypeWrapper):
+    """
+    Convert a serialized proto enum value to its name as a string.
+    Calls F.map_get(...) on the enum type's value-to-name mapping.
+
+    Parameters
+    ----------
+    value
+        The enum value
+    proto_enum
+        The class of the proto enum to convert
+
+    Examples
+    --------
+    >>> import chalk.functions as F
+    >>> from chalk.features import _, features
+    >>> from protos.gen.v1.transaction_pb2 import GetTransactionResponse, TransactionCategory
+    >>> @features
+    ... class Transaction:
+    ...    id: int
+    ...    transaction_response_bytes: bytes
+    ...    transaction_response: GetTransactionResponse = F.proto_deserialize(
+    ...        _.transaction_response_bytes,
+    ...        GetTransactionResponse,
+    ...    )
+    ...    transaction_category_name: F.proto_enum_value_to_name(_.transaction_response.category, TransactionCategory)
+
+    """
+    if not _is_protobuf_enum_type(proto_enum):
+        raise TypeError(
+            f"F.proto_enum_value_to_name(...) parameter 'proto_enum' must be a protobuf enum type class, instead got {proto_enum}"
+        )
+    from chalk import functions as F
+
+    mapping = {v: k for k, v in proto_enum.items()}
+    return F.map_get(mapping, value)
+
+
 def proto_serialize(mapping: Mapping[str, Union[Underscore, Any]], message: Message):
     """
     Serialize a proto message from a mapping of field names to values.
@@ -101,7 +147,11 @@ def proto_deserialize(body: Union[Underscore, bytes], message: Message):
     )
 
 
-__all__ = [
-    "proto_serialize",
-    "proto_deserialize",
-]
+def proto_timestamp_to_datetime(timestamp: Union[Underscore, timestamp_pb2.Timestamp]):
+    """
+    Converts a google.protobuf.timestamp.Timestamp object (a struct with 'seconds' & 'nanos' fields) into a UTC timestamp.
+    """
+    return UnderscoreFunction("from_unixtime", timestamp.seconds + (timestamp.nanos / 1_000_000_000.0))
+
+
+__all__ = ["proto_serialize", "proto_deserialize", "proto_enum_value_to_name", "proto_timestamp_to_datetime"]
