@@ -1,15 +1,26 @@
 from __future__ import annotations
 
+import ast
+import re
 import sys
 import types
 import typing
+from datetime import timedelta
 from typing import TYPE_CHECKING, Any, Dict, FrozenSet, List, Optional, Set, Tuple, Type, TypeVar, Union, cast
 
 from typing_extensions import Annotated, get_args, get_origin
 
 from chalk._lsp.error_builder import LSPErrorBuilder
+from chalk.features._encoding.http import HttpResponse
 from chalk.utils.cached_type_hints import cached_get_type_hints
 from chalk.utils.collections import is_optional
+from chalk.utils.import_utils import (
+    gather_all_imports_and_local_classes,
+    get_detailed_type_hint_errors,
+    import_only_type_checking_imports,
+)
+from chalk.utils.json import JSON
+from chalk.utils.log_with_context import get_logger
 
 if TYPE_CHECKING:
     from google.protobuf.message import Message as ProtobufMessage
@@ -18,6 +29,7 @@ if TYPE_CHECKING:
     from chalk.features import DataFrame, Feature, Features, Vector
     from chalk.streams import Windowed
 
+_logger = get_logger(__name__)
 
 T = TypeVar("T")
 U = TypeVar("U")
@@ -25,7 +37,6 @@ JsonValue = TypeVar("JsonValue")
 
 
 class ParsedAnnotation:
-
     __slots__ = (
         "_features_cls",
         "_attribute_name",
@@ -98,6 +109,8 @@ class ParsedAnnotation:
             typ = self._features_cls.__annotations__[self._attribute_name]
         else:
             typ = self._parsed_annotation or self._underlying or self._unparsed_underlying
+        if sys.version_info >= (3, 9) and get_origin(typ) == typing.Annotated:
+            typ = typ.__origin__  # pyright: ignore
         if isinstance(typ, type):
             typ = typ.__name__
         return str(typ)
@@ -111,9 +124,10 @@ class ParsedAnnotation:
         import typing
 
         import chalk
-        from chalk.features import FeatureSetBase
+        from chalk.features.feature_set import CURRENT_FEATURE_REGISTRY
 
-        feature_classes = {feature_set.__name__: feature_set for feature_set in FeatureSetBase.registry.values()}
+        registry = CURRENT_FEATURE_REGISTRY.get()
+        feature_classes = {feature_set.__name__: feature_set for feature_set in registry.get_feature_sets().values()}
 
         if not getattr(self._features_cls, "__chalk_is_loaded_from_notebook__", False):
             return feature_classes
@@ -141,14 +155,89 @@ class ParsedAnnotation:
                 # If there was an issue with that, then we'll fallback to trying to augment the globals, automatically injecting
                 # feature sets and chalkpy imports under their respective names
                 module = sys.modules.get(self._features_cls.__module__, None)
-                existing_globalns = None
+                existing_globalns = {}
                 if module is not None:
-                    existing_globalns = dict(getattr(module, "__dict__", {}))
-                    existing_globalns.update(self._get_globals_for_forward_references())
+                    existing_globalns.update(getattr(module, "__dict__", {}))
+                existing_globalns.update(self._get_globals_for_forward_references())
                 try:
-                    hints = cached_get_type_hints(self._features_cls, include_extras=True, globalns=existing_globalns)
-                except Exception as e:
-                    raise TypeError(f"Could not get type hints of {self._features_cls}") from e
+                    hints = cached_get_type_hints(
+                        self._features_cls,
+                        include_extras=True,
+                        globalns=existing_globalns if len(existing_globalns) > 0 else None,
+                    )
+                except:
+                    # If there was an issue with THAT, we'll try to import files specified by
+                    # TYPE_CHECKING only imports, and augment the globals again
+                    tree: ast.Module | None = None
+                    if self._features_cls.__chalk_source_info__.filename is not None:
+                        tree = import_only_type_checking_imports(self._features_cls.__chalk_source_info__.filename)
+                    existing_globalns.update(self._get_globals_for_forward_references())
+                    try:
+                        hints = cached_get_type_hints(
+                            self._features_cls,
+                            include_extras=True,
+                            globalns=existing_globalns if len(existing_globalns) > 0 else None,
+                        )
+                    except:
+                        # Last shot, in a notebook context, we may not have access to annotation types
+                        import datetime as dt
+                        from typing import Optional
+
+                        from chalk import Windowed
+                        from chalk.features import DataFrame, FeatureTime, Primary
+
+                        existing_globalns.update(
+                            {
+                                "Optional": Optional,
+                                "DataFrame": DataFrame,
+                                "Primary": Primary,
+                                "FeatureTime": FeatureTime,
+                                "dt": dt,
+                                "Windowed": Windowed,
+                            }
+                        )
+                        try:
+                            hints = cached_get_type_hints(
+                                self._features_cls,
+                                include_extras=True,
+                                globalns=existing_globalns if len(existing_globalns) > 0 else None,
+                            )
+                        except Exception as e:
+                            # At this point, we've failed. Let's try to get a better error message at the very least.
+                            name_error_pattern = re.compile(r"name '([^']*)' is not defined")
+                            attribute_errors = get_detailed_type_hint_errors(
+                                self._features_cls, True, existing_globalns if len(existing_globalns) > 0 else None
+                            )
+                            if isinstance(e, NameError) and tree is not None:
+                                feature_namespace = self._features_cls.namespace
+                                for attribute_name, error in attribute_errors.items():
+                                    match = name_error_pattern.search(str(error))
+                                    if match is not None:
+                                        missing_import_string = match.group(1)
+                                        (
+                                            regular_from_imports,
+                                            type_checking_imports,
+                                            local_classes,
+                                        ) = gather_all_imports_and_local_classes(tree)
+                                        if missing_import_string not in regular_from_imports + type_checking_imports:
+                                            if missing_import_string in local_classes:
+                                                # Feature class is defined somewhere else in the same file, so let's skip
+                                                continue
+                                            builder = self._features_cls and self._features_cls.__chalk_error_builder__
+                                            assert builder is not None
+                                            builder.add_diagnostic(
+                                                message=(
+                                                    f"Feature class '{feature_namespace}.{attribute_name}' has type annotation referencing '{missing_import_string}', "
+                                                    + f"which is undefined. Please import this using IS_TYPE_CHECKING. "
+                                                ),
+                                                label="invalid annotation",
+                                                code="79",
+                                                range=builder.annotation_range(attribute_name),
+                                            )
+                            raise TypeError(
+                                f"Could not get type hints of feature class '{self._features_cls}' from filename {self._features_cls.__chalk_source_info__.filename}: {str(e)}"
+                            ) from e
+
             parsed_annotation = hints[self._attribute_name]
         else:
             parsed_annotation = self._unparsed_underlying
@@ -160,7 +249,8 @@ class ParsedAnnotation:
                 # This is our catch-all, which we only want to run if someone else didn't already report an error.
                 not LSPErrorBuilder.has_errors()
                 and self._features_cls is not None
-                and self._features_cls.__chalk_error_builder__ is not None
+                and self._features_cls.__chalk_error_builder__
+                is not None  # pyright: ignore[reportUnnecessaryComparison]
             ):
                 try:
                     self._type_error(message=e.args[0], code="79")
@@ -201,7 +291,7 @@ class ParsedAnnotation:
         except ImportError:
             ProtobufMessage = None
 
-        assert self._parsed_annotation is None, "The annotation was already parsed"
+        # assert self._parsed_annotation is None, "The annotation was already parsed"
         if isinstance(annotation, Windowed):
             # If it's windowed, then unwrap it immediately, because Windowed annotations are really just a proxy to the underlying type
             annotation = annotation.kind
@@ -216,7 +306,9 @@ class ParsedAnnotation:
             error_ctx = ""
         origin = get_origin(annotation)
 
-        if origin in (
+        if annotation is JSON:
+            self._is_nullable = True
+        elif origin in (
             Union,
             getattr(types, "UnionType", Union),
         ):  # using getattr as UnionType was introduced in python 3.10
@@ -230,6 +322,18 @@ class ParsedAnnotation:
                 )
 
             annotation = args[0] if args[1] in (None, type(None)) else args[1]
+            if isinstance(annotation, type) and issubclass(annotation, DataFrame):
+                self._type_error(
+                    (
+                        f"Invalid type annotation for feature '{error_ctx}': "
+                        f"'{annotation}' refers to an Optional DataFrame. "
+                        "Since DataFrames can be empty, the Optional is unnecessary. "
+                        "Please refer to your DataFrame as a non-optional type, e.g. `DataFrame[FeatureClassB]`."
+                    ),
+                    code="78",
+                    label="invalid annotation",
+                )
+                return
             origin = get_origin(annotation)
             self._is_nullable = True
 
@@ -303,7 +407,7 @@ class ParsedAnnotation:
                 )
             if args[1] in (Any, object) or (get_origin(args[1]) == Union and not is_optional(args[1])):
                 self._type_error(
-                    f"Only homogeneous dicts supported, found `{Dict[args]}",  # pyright: ignore
+                    f"Only homogeneous dicts supported, found `{Dict[args]}`",  # pyright: ignore
                     code="158",
                     label="`Any` as dict value type not allowed",
                 )
@@ -314,7 +418,11 @@ class ParsedAnnotation:
             # We never want FeatureWrappers; if this is the case, then unwrap it to the underlying feature
             annotation = unwrap_feature(annotation)
 
-        if not isinstance(annotation, (type, Feature)):
+        if not isinstance(annotation, (type, Feature)) and annotation not in (
+            JSON,
+            HttpResponse[str],
+            HttpResponse[bytes],
+        ):
             if isinstance(annotation, str):
                 self._type_error(
                     (
@@ -344,14 +452,14 @@ class ParsedAnnotation:
                     (
                         f"Invalid type annotation for feature '{error_ctx}': "
                         f"'{self._parsed_annotation}' does not reference a Python type, Chalk feature, "
-                        f"or a type annotation."
+                        "or a type annotation."
                     ),
                     code="78",
                     label="invalid annotation",
                 )
             return
 
-        self._underlying = annotation
+        self._underlying = annotation  # pyright: ignore[reportAttributeAccessIssue]
         if isinstance(self._underlying, type):
             if ProtobufMessage is not None and issubclass(self._underlying, ProtobufMessage):
                 self._as_proto = self._underlying
@@ -360,11 +468,42 @@ class ParsedAnnotation:
                 self._as_features_cls = self._underlying
 
             elif issubclass(self._underlying, DataFrame):
+                if len(self._underlying.columns) == 0:
+                    self._type_error(
+                        (
+                            f"Invalid type annotation for feature '{error_ctx}': "
+                            f"'{self._parsed_annotation}' does not reference a Chalk feature class. "
+                            "Please add your feature class to the type, e.g. `DataFrame[FeatureClassB]`. "
+                        ),
+                        code="78",
+                        label="invalid annotation",
+                    )
+                    return
+                if self._features_cls is not None and self._underlying.namespace == self._features_cls.namespace:
+                    self._type_error(
+                        (
+                            f"Invalid type annotation for feature '{error_ctx}': "
+                            f"'{self._parsed_annotation}' is a DataFrame that refers to its own feature class. "
+                            "This is unsupported."
+                        ),
+                        code="78",
+                        label="invalid annotation",
+                    )
+                    return
+
                 self._as_dataframe = self._underlying
 
             elif issubclass(self._underlying, Vector):
                 self._as_vector = self._underlying
 
+            elif issubclass(self._underlying, timedelta):
+                self._type_error(
+                    (f"Invalid type annotation for feature '{error_ctx}': timedelta feature types are not supported."),
+                    code="78",
+                    label="invalid annotation",
+                )
+                return
+
         elif isinstance(self._underlying, Feature):
             self._as_feature = self._underlying
 
